@@ -13,16 +13,17 @@
 
 (macrolet ((def (name result access src-type &optional typep)
              `(defun ,name (object ,@(if typep '(type) ()))
+                (declare (type ,(ecase src-type
+                                       (:list 'list)
+                                       (:vector 'vector)
+                                       (:sequence 'sequence)) object))
                 (do* ((index 0 (1+ index))
-                      (length (length (the ,(ecase src-type
-                                              (:list 'list)
-                                              (:vector 'vector)
-                                              (:sequence 'sequence))
-                                           object)))
+                      (length (length object))
                       (result ,result)
                       (in-object object))
-                     ((= index length) result)
+                     ((>= index length) result)
                   (declare (fixnum length index))
+                  (declare (type vector result))
                   (setf (,access result index)
                         ,(ecase src-type
                            (:list '(pop in-object))
@@ -39,16 +40,24 @@
     aref :sequence t))
 
 (defun vector-to-list* (object)
+  (declare (type vector object))
   (let ((result (list nil))
         (length (length object)))
     (declare (fixnum length))
     (do ((index 0 (1+ index))
          (splice result (cdr splice)))
-        ((= index length) (cdr result))
+        ((>= index length) (cdr result))
       (declare (fixnum index))
       (rplacd splice (list (aref object index))))))
 
-(defvar *offending-datum*); FIXME: Remove after debugging COERCE.
+(defun sequence-to-list (sequence)
+  (declare (type sequence sequence))
+  (let* ((result (list nil))
+         (splice result))
+    (sb!sequence:dosequence (i sequence)
+      (rplacd splice (list i))
+      (setf splice (cdr splice)))
+    (cdr result)))
 
 ;;; These are used both by the full DEFUN function and by various
 ;;; optimization transforms in the constant-OUTPUT-TYPE-SPEC case.
@@ -58,48 +67,52 @@
 ;;; DEFTRANSFORMs, though.
 (declaim (inline coerce-to-list))
 (declaim (inline coerce-to-vector))
+
+(defun coerce-symbol-to-fun (object)
+  (let ((kind (info :function :kind object)))
+    (case kind
+      (:macro
+       (error "~S names a macro." object))
+      (:special-form
+       (error "~S names a special operator." object))
+      (t
+       (fdefinition object)))))
+
 (defun coerce-to-fun (object)
   ;; (Unlike the other COERCE-TO-FOOs, this one isn't inline, because
   ;; it's so big and because optimizing away the outer ETYPECASE
   ;; doesn't seem to buy us that much anyway.)
   (etypecase object
+    (function object)
     (symbol
-     ;; ANSI lets us return ordinary errors (non-TYPE-ERRORs) here.
-     (cond ((macro-function object)
-            (error "~S names a macro." object))
-           ((special-operator-p object)
-            (error "~S is a special operator." object))
-           (t (fdefinition object))))
+     (coerce-symbol-to-fun object))
     (list
      (case (first object)
-       ((setf)
+       (setf
         (fdefinition object))
-       ((lambda)
-        ;; FIXME: If we go to a compiler-only implementation, this can
-        ;; become COMPILE instead of EVAL, which seems nicer to me.
-        (eval `(function ,object)))
-       ((instance-lambda)
-        (deprecation-warning 'instance-lambda 'lambda)
-        (eval `(function ,object)))
+       (lambda
+        (eval object))
        (t
         (error 'simple-type-error
                :datum object
                :expected-type '(or symbol
-                                   ;; KLUDGE: ANSI wants us to
-                                   ;; return a TYPE-ERROR here, and
-                                   ;; a TYPE-ERROR is supposed to
-                                   ;; describe the expected type,
-                                   ;; but it's not obvious how to
-                                   ;; describe the coerceable cons
-                                   ;; types, so we punt and just say
-                                   ;; CONS. -- WHN 20000503
-                                   cons)
+                                ;; KLUDGE: ANSI wants us to
+                                ;; return a TYPE-ERROR here, and
+                                ;; a TYPE-ERROR is supposed to
+                                ;; describe the expected type,
+                                ;; but it's not obvious how to
+                                ;; describe the coerceable cons
+                                ;; types, so we punt and just say
+                                ;; CONS. -- WHN 20000503
+                                cons)
                :format-control "~S can't be coerced to a function."
                :format-arguments (list object)))))))
 
 (defun coerce-to-list (object)
-  (etypecase object
-    (vector (vector-to-list* object))))
+  (seq-dispatch object
+                object
+                (vector-to-list* object)
+                (sequence-to-list object)))
 
 (defun coerce-to-vector (object output-type-spec)
   (etypecase object
@@ -123,7 +136,7 @@
          object)
         ((eq type *empty-type*)
          (coerce-error))
-        ((csubtypep type (specifier-type 'character))
+        ((type= type (specifier-type 'character))
          (character object))
         ((numberp object)
          (cond
@@ -164,7 +177,7 @@
                      ((csubtypep type (specifier-type '(complex float)))
                       (complex (%single-float (realpart object))
                                (%single-float (imagpart object))))
-                     ((and (typep object 'rational)
+                     ((and (typep object 'rational) ; TODO jmoringe unreachable?
                            (csubtypep type (specifier-type '(complex float))))
                       ;; Perhaps somewhat surprisingly, ANSI specifies
                       ;; that (COERCE FOO 'FLOAT) is a SINGLE-FLOAT,
@@ -242,37 +255,13 @@
         ((and (csubtypep type (specifier-type 'sequence))
               (find-class output-type-spec nil))
          (let ((class (find-class output-type-spec)))
+           (unless (sb!mop:class-finalized-p class)
+             (sb!mop:finalize-inheritance class))
            (sb!sequence:make-sequence-like
             (sb!mop:class-prototype class)
             (length object) :initial-contents object)))
         ((csubtypep type (specifier-type 'function))
-         (when (and (legal-fun-name-p object)
-                    (not (fboundp object)))
-           (error 'simple-type-error
-                  :datum object
-                  ;; FIXME: SATISFIES FBOUNDP is a kinda bizarre broken
-                  ;; type specifier, since the set of values it describes
-                  ;; isn't in general constant in time. Maybe we could
-                  ;; find a better way of expressing this error? (Maybe
-                  ;; with the UNDEFINED-FUNCTION condition?)
-                  :expected-type '(satisfies fboundp)
-               :format-control "~S isn't fbound."
-               :format-arguments (list object)))
-         (when (and (symbolp object)
-                    (sb!xc:macro-function object))
-           (error 'simple-type-error
-                  :datum object
-                  :expected-type '(not (satisfies sb!xc:macro-function))
-                  :format-control "~S is a macro."
-                  :format-arguments (list object)))
-         (when (and (symbolp object)
-                    (special-operator-p object))
-           (error 'simple-type-error
-                  :datum object
-                  :expected-type '(not (satisfies special-operator-p))
-                  :format-control "~S is a special operator."
-                  :format-arguments (list object)))
-         (eval `#',object))
+         (coerce-to-fun object))
         (t
          (coerce-error))))))
 
