@@ -365,8 +365,8 @@
                          (condition-slot-initform-p sslot))
                    (setf (condition-slot-initform found)
                          (condition-slot-initform sslot))
-                   (setf (condition-slot-initfunction sslot)
-                         (condition-slot-initfunction found)))
+                   (setf (condition-slot-initfunction found)
+                         (condition-slot-initfunction sslot)))
                  (unless (condition-slot-allocation found)
                    (setf (condition-slot-allocation found)
                          (condition-slot-allocation sslot))))
