@@ -1851,9 +1851,7 @@ variable: an unreadable object representing the error is printed instead.")
   nil)
 
 (defun output-fun (object stream)
-  (let* ((*print-length* 4)  ; in case we have to..
-         (*print-level* 3)  ; ..print an interpreted function definition
-         (name (%fun-name object))
+  (let* ((name (%fun-name object))
          (proper-name-p (and (legal-fun-name-p name) (fboundp name)
                              (eq (fdefinition name) object))))
     (print-unreadable-object (object stream :identity (not proper-name-p))
