@@ -27,9 +27,13 @@
   (xmm-args 0)
   (stack-frame-size 0))
 
+(defconstant max-int-args #.(length *c-call-register-arg-offsets*))
+(defconstant max-xmm-args #!+win32 4 #!-win32 8)
+
 (defun int-arg (state prim-type reg-sc stack-sc)
-  (let ((reg-args (arg-state-register-args state)))
-    (cond ((< reg-args 6)
+  (let ((reg-args (max (arg-state-register-args state)
+                       #!+win32 (arg-state-xmm-args state))))
+    (cond ((< reg-args max-int-args)
            (setf (arg-state-register-args state) (1+ reg-args))
            (my-make-wired-tn prim-type reg-sc
                              (nth reg-args *c-call-register-arg-offsets*)))
@@ -48,8 +52,9 @@
   (int-arg state 'system-area-pointer 'sap-reg 'sap-stack))
 
 (defun float-arg (state prim-type reg-sc stack-sc)
-  (let ((xmm-args (arg-state-xmm-args state)))
-    (cond ((< xmm-args 8)
+  (let ((xmm-args (max (arg-state-xmm-args state)
+                        #!+win32 (arg-state-register-args state))))
+    (cond ((< xmm-args max-xmm-args)
            (setf (arg-state-xmm-args state) (1+ xmm-args))
            (my-make-wired-tn prim-type reg-sc
                              (nth xmm-args *float-regs*)))
@@ -74,8 +79,6 @@
     (0 eax-offset)
     (1 edx-offset)))
 
-;; XXX The return handling probably doesn't conform to the ABI
-
 (define-alien-type-method (integer :result-tn) (type state)
   (let ((num-results (result-state-num-results state)))
     (setf (result-state-num-results state) (1+ num-results))
@@ -86,9 +89,10 @@
       (my-make-wired-tn ptype reg-sc (result-reg-offset num-results)))))
 
 (define-alien-type-method (integer :naturalize-gen) (type alien)
-  (if (and (alien-integer-type-signed type)
-           (<= (alien-type-bits type) 32))
-      `(sign-extend ,alien)
+  (if (<= (alien-type-bits type) 32)
+      (if (alien-integer-type-signed type)
+          `(sign-extend ,alien ,(alien-type-bits type))
+          `(logand ,alien ,(1- (ash 1 (alien-type-bits type)))))
       alien))
 
 (define-alien-type-method (system-area-pointer :result-tn) (type state)
@@ -118,7 +122,7 @@
               (invoke-alien-type-method :result-tn type state))
             values)))
 
-(!def-vm-support-routine make-call-out-tns (type)
+(defun make-call-out-tns (type)
   (let ((arg-state (make-arg-state)))
     (collect ((arg-tns))
       (dolist (arg-type (alien-fun-type-arg-types type))
@@ -191,34 +195,43 @@
                                     ,@(new-args))))))
         (sb!c::give-up-ir1-transform))))
 
-;;; The ABI specifies that signed short/int's are returned as 32-bit
-;;; values. Negative values need to be sign-extended to 64-bits (done
-;;; in a :NATURALIZE-GEN alien-type-method).
-(defknown sign-extend ((signed-byte 32)) fixnum
-          (foldable flushable movable))
+;;; The ABI is vague about how signed sub-word integer return values
+;;; are handled, but since gcc versions >=4.3 no longer do sign
+;;; extension in the callee, we need to do it in the caller.  FIXME:
+;;; If the value to be extended is known to already be of the target
+;;; type at compile time, we can (and should) elide the extension.
+(defknown sign-extend ((signed-byte 64) t) fixnum
+    (foldable flushable movable))
 
 (define-vop (sign-extend)
   (:translate sign-extend)
   (:policy :fast-safe)
   (:args (val :scs (signed-reg)))
-  (:arg-types fixnum)
+  (:arg-types signed-num (:constant fixnum))
+  (:info size)
   (:results (res :scs (signed-reg)))
   (:result-types fixnum)
   (:generator 1
    (inst movsxd res
          (make-random-tn :kind :normal
-                         :sc (sc-or-lose 'dword-reg)
+                         :sc (sc-or-lose (ecase size
+                                           (8 'byte-reg)
+                                           (16 'word-reg)
+                                           (32 'dword-reg)))
                          :offset (tn-offset val)))))
 
 #-sb-xc-host
-(defun sign-extend (x)
-  (declare (type (signed-byte 32) x))
-  (sign-extend x))
+(defun sign-extend (x size)
+  (declare (type (signed-byte 64) x))
+  (ecase size
+    (8 (sign-extend x size))
+    (16 (sign-extend x size))
+    (32 (sign-extend x size))))
 
 #+sb-xc-host
-(defun sign-extend (x)
-  (if (logbitp 31 x)
-      (dpb x (byte 32 0) -1)
+(defun sign-extend (x size)
+  (if (logbitp (1- size) x)
+      (dpb x (byte size 0) -1)
       x))
 
 (define-vop (foreign-symbol-sap)
@@ -230,7 +243,7 @@
   (:results (res :scs (sap-reg)))
   (:result-types system-area-pointer)
   (:generator 2
-   (inst lea res (make-fixup foreign-symbol :foreign))))
+   (inst mov res (make-fixup foreign-symbol :foreign))))
 
 #!+linkage-table
 (define-vop (foreign-symbol-dataref-sap)
@@ -245,30 +258,78 @@
    (inst mov res (make-fixup foreign-symbol :foreign-dataref))))
 
 (define-vop (call-out)
-  (:args (function :scs (sap-reg))
+  (:args (function :scs (sap-reg)
+                   :target rbx)
          (args :more t))
   (:results (results :more t))
+  ;; RBX is used to first load the address, allowing the debugger to
+  ;; determine which alien was accessed in case it's undefined.
+  (:temporary (:sc sap-reg :offset rbx-offset) rbx)
   (:temporary (:sc unsigned-reg :offset rax-offset :to :result) rax)
-  (:ignore results)
+  ;; For safepoint builds: Force values of non-volatiles to the stack.
+  ;; These are the callee-saved registers in the native ABI, but
+  ;; safepoint-based GC needs to see all Lisp values on the stack.  Note
+  ;; that R12-R15 are non-volatile registers, but there is no need to
+  ;; spill R12 because it is our thread-base-tn.  RDI and RSI are
+  ;; non-volatile on Windows, but argument passing registers on other
+  ;; platforms.
+  #!+sb-safepoint (:temporary (:sc unsigned-reg :offset r13-offset) r13)
+  #!+sb-safepoint (:temporary (:sc unsigned-reg :offset r14-offset) r14)
+  #!+sb-safepoint (:temporary (:sc unsigned-reg :offset r15-offset) r15)
+  #!+(and sb-safepoint win32) (:temporary
+                               (:sc unsigned-reg :offset rdi-offset) rdi)
+  #!+(and sb-safepoint win32) (:temporary
+                               (:sc unsigned-reg :offset rsi-offset) rsi)
+  (:ignore results
+           #!+(and sb-safepoint win32) rdi
+           #!+(and sb-safepoint win32) rsi
+           #!+win32 args
+           #!+win32 rax
+           #!+sb-safepoint r15
+           #!+sb-safepoint r13)
   (:vop-var vop)
   (:save-p t)
   (:generator 0
     ;; ABI: Direction flag must be clear on function entry. -- JES, 2006-01-20
     (inst cld)
+    #!+sb-safepoint
+    (progn
+      ;; Current PC - don't rely on function to keep it in a form that
+      ;; GC understands
+      (let ((label (gen-label)))
+        (inst lea r14 (make-fixup nil :code-object label))
+        (emit-label label)))
+    #!-win32
     ;; ABI: AL contains amount of arguments passed in XMM registers
     ;; for vararg calls.
     (move-immediate rax
                     (loop for tn-ref = args then (tn-ref-across tn-ref)
-                       while tn-ref
-                       count (eq (sb-name (sc-sb (tn-sc (tn-ref-tn tn-ref))))
-                                 'float-registers)))
-    (inst call function)
+                          while tn-ref
+                          count (eq (sb-name (sc-sb (tn-sc (tn-ref-tn tn-ref))))
+                                    'float-registers)))
+    #!+win32 (inst sub rsp-tn #x20) ;MS_ABI: shadow zone
+    #!+sb-safepoint
+    (progn                 ;Store SP and PC in thread struct
+      (storew rsp-tn thread-base-tn thread-saved-csp-offset)
+      (storew r14 thread-base-tn thread-pc-around-foreign-call-slot))
+    (move rbx function)
+    (inst call rbx)
+    #!+win32 (inst add rsp-tn #x20) ;MS_ABI: remove shadow space
+    #!+sb-safepoint
+    (progn
+      ;; Zeroing out
+      (inst xor r14 r14)
+      ;; Zero PC storage place. NB. CSP-then-PC: same sequence on
+      ;; entry/exit, is actually corrent.
+      (storew r14 thread-base-tn thread-saved-csp-offset)
+      (storew r14 thread-base-tn thread-pc-around-foreign-call-slot))
     ;; To give the debugger a clue. XX not really internal-error?
     (note-this-location vop :internal-error)))
 
 (define-vop (alloc-number-stack-space)
   (:info amount)
   (:results (result :scs (sap-reg any-reg)))
+  (:result-types system-area-pointer)
   (:generator 0
     (aver (location= result rsp-tn))
     (unless (zerop amount)
@@ -285,62 +346,19 @@
       (let ((delta (logandc2 (+ amount 7) 7)))
         (inst add rsp-tn delta)))))
 
-(define-vop (alloc-alien-stack-space)
-  (:info amount)
-  #!+sb-thread (:temporary (:sc unsigned-reg) temp)
-  (:results (result :scs (sap-reg any-reg)))
-  #!+sb-thread
-  (:generator 0
-    (aver (not (location= result rsp-tn)))
-    (unless (zerop amount)
-      (let ((delta (logandc2 (+ amount 7) 7)))
-        (inst mov temp
-              (make-ea :qword
-                       :disp (+ nil-value
-                                (static-symbol-offset '*alien-stack*)
-                                (ash symbol-tls-index-slot word-shift)
-                                (- other-pointer-lowtag))))
-        (inst sub (make-ea :qword :base thread-base-tn
-                           :scale 1 :index temp) delta)))
-    (load-tl-symbol-value result *alien-stack*))
-  #!-sb-thread
-  (:generator 0
-    (aver (not (location= result rsp-tn)))
-    (unless (zerop amount)
-      (let ((delta (logandc2 (+ amount 7) 7)))
-        (inst sub (make-ea :qword
-                           :disp (+ nil-value
-                                    (static-symbol-offset '*alien-stack*)
-                                    (ash symbol-value-slot word-shift)
-                                    (- other-pointer-lowtag)))
-              delta)))
-    (load-symbol-value result *alien-stack*)))
-
-(define-vop (dealloc-alien-stack-space)
-  (:info amount)
-  #!+sb-thread (:temporary (:sc unsigned-reg) temp)
-  #!+sb-thread
-  (:generator 0
-    (unless (zerop amount)
-      (let ((delta (logandc2 (+ amount 7) 7)))
-        (inst mov temp
-              (make-ea :qword
-                       :disp (+ nil-value
-                                (static-symbol-offset '*alien-stack*)
-                                (ash symbol-tls-index-slot word-shift)
-                                (- other-pointer-lowtag))))
-        (inst add (make-ea :qword :base thread-base-tn :scale 1 :index temp)
-              delta))))
-  #!-sb-thread
-  (:generator 0
-    (unless (zerop amount)
-      (let ((delta (logandc2 (+ amount 7) 7)))
-        (inst add (make-ea :qword
-                           :disp (+ nil-value
-                                    (static-symbol-offset '*alien-stack*)
-                                    (ash symbol-value-slot word-shift)
-                                    (- other-pointer-lowtag)))
-              delta)))))
+(macrolet ((alien-stack-ptr ()
+             #!+sb-thread '(symbol-known-tls-cell '*alien-stack-pointer*)
+             #!-sb-thread '(static-symbol-value-ea '*alien-stack-pointer*)))
+  (define-vop (alloc-alien-stack-space)
+    (:info amount)
+    (:results (result :scs (sap-reg any-reg)))
+    (:result-types system-area-pointer)
+    (:generator 0
+      (aver (not (location= result rsp-tn)))
+      (unless (zerop amount)
+        (let ((delta (logandc2 (+ amount 7) 7)))
+          (inst sub (alien-stack-ptr) delta)))
+      (inst mov result (alien-stack-ptr)))))
 
 ;;; not strictly part of the c-call convention, but needed for the
 ;;; WITH-PINNED-OBJECTS macro used for "locking down" lisp objects so
@@ -370,23 +388,24 @@
              (error "Too many arguments in callback")))
     (let* ((segment (make-segment))
            (rax rax-tn)
-           (rcx rcx-tn)
-           (rdi rdi-tn)
-           (rsi rsi-tn)
+           #!+(or win32 (not sb-safepoint)) (rcx rcx-tn)
+           #!-win32 (rdi rdi-tn)
+           #!-win32 (rsi rsi-tn)
            (rdx rdx-tn)
            (rbp rbp-tn)
            (rsp rsp-tn)
+           #!+win32 (r8 r8-tn)
            (xmm0 float0-tn)
            ([rsp] (make-ea :qword :base rsp :disp 0))
            ;; How many arguments have been copied
            (arg-count 0)
            ;; How many arguments have been copied from the stack
-           (stack-argument-count 0)
+           (stack-argument-count #!-win32 0 #!+win32 4)
            (gprs (mapcar (make-tn-maker 'any-reg) *c-call-register-arg-offsets*))
            (fprs (mapcar (make-tn-maker 'double-reg)
                          ;; Only 8 first XMM registers are used for
                          ;; passing arguments
-                         (subseq *float-regs* 0 8))))
+                         (subseq *float-regs* 0 #!-win32 8 #!+win32 4))))
       (assemble (segment)
         ;; Make room on the stack for arguments.
         (inst sub rsp (* n-word-bytes (length argument-types)))
@@ -409,6 +428,7 @@
             (incf arg-count)
             (cond (integerp
                    (let ((gpr (pop gprs)))
+                     #!+win32 (pop fprs)
                      ;; Argument not in register, copy it from the old
                      ;; stack location to a temporary register.
                      (unless gpr
@@ -421,6 +441,7 @@
                   ((or (alien-single-float-type-p type)
                        (alien-double-float-type-p type))
                    (let ((fpr (pop fprs)))
+                     #!+win32 (pop gprs)
                      (cond (fpr
                             ;; Copy from float register to target location.
                             (inst movq target-tn fpr))
@@ -434,36 +455,60 @@
                   (t
                    (bug "Unknown alien floating point type: ~S" type)))))
 
-        ;; arg0 to FUNCALL3 (function)
-        ;;
-        ;; Indirect the access to ENTER-ALIEN-CALLBACK through
-        ;; the symbol-value slot of SB-ALIEN::*ENTER-ALIEN-CALLBACK*
-        ;; to ensure it'll work even if the GC moves ENTER-ALIEN-CALLBACK.
-        ;; Skip any SB-THREAD TLS magic, since we don't expect anyone
-        ;; to rebind the variable. -- JES, 2006-01-01
-        (inst mov rdi (+ nil-value (static-symbol-offset
-                                    'sb!alien::*enter-alien-callback*)))
-        (loadw rdi rdi symbol-value-slot other-pointer-lowtag)
-        ;; arg0 to ENTER-ALIEN-CALLBACK (trampoline index)
-        (inst mov rsi (fixnumize index))
-        ;; arg1 to ENTER-ALIEN-CALLBACK (pointer to argument vector)
-        (inst mov rdx rsp)
-        ;; add room on stack for return value
-        (inst sub rsp 8)
-        ;; arg2 to ENTER-ALIEN-CALLBACK (pointer to return value)
-        (inst mov rcx rsp)
+        #!-sb-safepoint
+        (progn
+          ;; arg0 to FUNCALL3 (function)
+          ;;
+          ;; Indirect the access to ENTER-ALIEN-CALLBACK through
+          ;; the symbol-value slot of SB-ALIEN::*ENTER-ALIEN-CALLBACK*
+          ;; to ensure it'll work even if the GC moves ENTER-ALIEN-CALLBACK.
+          ;; Skip any SB-THREAD TLS magic, since we don't expect anyone
+          ;; to rebind the variable. -- JES, 2006-01-01
+          (inst mov rdi (+ nil-value (static-symbol-offset
+                                      'sb!alien::*enter-alien-callback*)))
+          (loadw rdi rdi symbol-value-slot other-pointer-lowtag)
+          ;; arg0 to ENTER-ALIEN-CALLBACK (trampoline index)
+          (inst mov rsi (fixnumize index))
+          ;; arg1 to ENTER-ALIEN-CALLBACK (pointer to argument vector)
+          (inst mov rdx rsp)
+          ;; add room on stack for return value
+          (inst sub rsp 8)
+          ;; arg2 to ENTER-ALIEN-CALLBACK (pointer to return value)
+          (inst mov rcx rsp)
 
-        ;; Make new frame
-        (inst push rbp)
-        (inst mov  rbp rsp)
+          ;; Make new frame
+          (inst push rbp)
+          (inst mov  rbp rsp)
 
-        ;; Call
-        (inst mov  rax (foreign-symbol-address "funcall3"))
-        (inst call rax)
+          ;; Call
+          (inst mov  rax (foreign-symbol-address "funcall3"))
+          (inst call rax)
 
-        ;; Back! Restore frame
-        (inst mov rsp rbp)
-        (inst pop rbp)
+          ;; Back! Restore frame
+          (inst mov rsp rbp)
+          (inst pop rbp))
+
+        #!+sb-safepoint
+        (progn
+          ;; arg0 to ENTER-ALIEN-CALLBACK (trampoline index)
+          (inst mov #!-win32 rdi #!+win32 rcx (fixnumize index))
+          ;; arg1 to ENTER-ALIEN-CALLBACK (pointer to argument vector)
+          (inst mov #!-win32 rsi #!+win32 rdx rsp)
+          ;; add room on stack for return value
+          (inst sub rsp 8)
+          ;; arg2 to ENTER-ALIEN-CALLBACK (pointer to return value)
+          (inst mov #!-win32 rdx #!+win32 r8 rsp)
+          ;; Make new frame
+          (inst push rbp)
+          (inst mov  rbp rsp)
+          #!+win32 (inst sub rsp #x20)
+          #!+win32 (inst and rsp #x-20)
+          ;; Call
+          (inst mov rax (foreign-symbol-address "callback_wrapper_trampoline"))
+          (inst call rax)
+          ;; Back! Restore frame
+          (inst mov rsp rbp)
+          (inst pop rbp))
 
         ;; Result now on top of stack, put it in the right register
         (cond
