@@ -91,12 +91,12 @@ dyndebug_init()
         for (;;) {
             char *token = strtok(ptr, " ");
             if (!token) break;
-            unsigned i;
+            int i;
             if (!strcmp(token, "all"))
                 for (i = 0; i < n_output_flags; i++)
                     *ptrs[i] = 1;
             else {
-                for (i = 0; i < DYNDEBUG_NFLAGS; i++)
+                for (i = 0; i < (int)DYNDEBUG_NFLAGS; i++)
                     if (!strcmp(token, names[i])) {
                         *ptrs[i] = 1;
                         break;
@@ -112,8 +112,8 @@ dyndebug_init()
         if (err) {
             fprintf(stderr, "Valid flags are:\n");
             fprintf(stderr, "  all  ;enables all of the following:\n");
-            unsigned i;
-            for (i = 0; i < DYNDEBUG_NFLAGS; i++) {
+            int i;
+            for (i = 0; i < (int)DYNDEBUG_NFLAGS; i++) {
                 if (i == n_output_flags)
                     fprintf(stderr, "Additional options:\n");
                 fprintf(stderr, "  %s\n", names[i]);
@@ -300,9 +300,9 @@ static void brief_fixnum(lispobj obj)
     if (!fixnump(obj)) return print_unknown(obj);
 
 #ifndef LISP_FEATURE_ALPHA
-    printf("%ld", ((long)obj)>>2);
+    printf("%ld", ((long)obj)>>N_FIXNUM_TAG_BITS);
 #else
-    printf("%d", ((s32)obj)>>2);
+    printf("%d", ((s32)obj)>>N_FIXNUM_TAG_BITS);
 #endif
 }
 
@@ -314,9 +314,9 @@ static void print_fixnum(lispobj obj)
     if (!fixnump(obj)) return print_unknown(obj);
 
 #ifndef LISP_FEATURE_ALPHA
-    printf(": %ld", ((long)obj)>>2);
+    printf(": %ld", ((long)obj)>>N_FIXNUM_TAG_BITS);
 #else
-    printf(": %d", ((s32)obj)>>2);
+    printf(": %d", ((s32)obj)>>N_FIXNUM_TAG_BITS);
 #endif
 }
 
@@ -502,6 +502,30 @@ static void brief_otherptr(lispobj obj)
             putchar('"');
             break;
 
+#ifdef LISP_FEATURE_SB_UNICODE
+        case SIMPLE_CHARACTER_STRING_WIDETAG:
+            vector = (struct vector *)ptr;
+            fputs("u\"", stdout);
+            {
+                int i, ch, len = fixnum_value(vector->length);
+                uint32_t *chars = (uint32_t*)vector->data;
+                for (i=0 ; i<len ; i++) {
+                    ch = chars[i];
+                    if (ch >= 32 && ch < 127) {
+                        if (ch == '"' || ch == '\\')
+                            putchar('\\');
+                        putchar(ch);
+                    } else {
+                      // ambiguous, e.g. #\xaaa is either #\GUJARATI_LETTER_PA
+                      // or #\FEMININE_ORDINAL_INDICATOR + #\a. oh well.
+                      printf("\\x%x", ch);
+                    }
+                }
+            }
+            putchar('"');
+            break;
+#endif
+
         default:
             printf("#<ptr to ");
             brief_otherimm(header);
@@ -525,7 +549,7 @@ static void print_slots(char **slots, int count, lispobj *ptr)
  * sbcl.h). */
 static char *symbol_slots[] = {"value: ", "hash: ",
     "info: ", "name: ", "package: ",
-#ifdef LISP_FEATURE_SB_THREAD
+#if defined (LISP_FEATURE_SB_THREAD) && !defined(LISP_FEATURE_X86_64)
     "tls-index: " ,
 #endif
     NULL};
@@ -593,7 +617,9 @@ static void print_otherptr(lispobj obj)
                 break;
 
             case SYMBOL_HEADER_WIDETAG:
-                print_slots(symbol_slots, count, ptr);
+                // Only 1 byte of a symbol header conveys its size.
+                // The other bytes may be freely used by the backend.
+                print_slots(symbol_slots, count & 0xFF, ptr);
                 break;
 
 #if N_WORD_BITS == 32
