@@ -125,4 +125,71 @@
   (funcall fun)
   (assert (equal '(:ok) (read-from-string "{:ok)"))))
 
+(with-test (:name :bad-recursive-read)
+  ;; This use to signal an unbound-variable error instead.
+  (assert (eq :error
+              (handler-case
+                  (with-input-from-string (s "42")
+                    (read s t nil t))
+                (reader-error (e)
+                  :error)))))
+
+(with-test (:name :standard-readtable-modified)
+  (macrolet ((test (form &optional op)
+               `(assert
+                 (eq :error
+                     (handler-case
+                         (progn ,form t)
+                       (sb-int:standard-readtable-modified-error (e)
+                         ,@(when op
+                            `((assert
+                               (equal ,op (sb-kernel::standard-readtable-modified-operation e)))))
+                         :error))))))
+    (let ((rt *readtable*))
+     (with-standard-io-syntax
+       (let ((srt *readtable*))
+         (test (setf (readtable-case srt) :preserve) '(setf readtable-case))
+         (test (copy-readtable rt srt) 'copy-readtable)
+         (test (set-syntax-from-char #\a #\a srt rt) 'set-syntax-from-char)
+         (test (set-macro-character #\a (constantly t) t srt) 'set-macro-character)
+         (test (make-dispatch-macro-character #\! t srt))
+         (test (set-dispatch-macro-character #\# #\a (constantly t) srt) 'set-dispatch-macro-character))))))
+
+(with-test (:name :reader-package-errors)
+  (flet ((test (string)
+           (handler-case
+               (progn (read-from-string string) :feh)
+             (error (e)
+               (when (and (typep e 'reader-error) (typep e 'package-error))
+                 (package-error-package e))))))
+    (assert (equal "NO-SUCH-PKG" (test "no-such-pkg::foo")))
+    (assert (eq (find-package :cl) (test "cl:no-such-sym")))))
+
+;;; THIS SHOULD BE LAST as it frobs the standard readtable
+(with-test (:name :set-macro-character-nil)
+  (handler-bind ((sb-int:standard-readtable-modified-error #'continue))
+    (let ((fun (lambda (&rest args) 'ok)))
+      ;; NIL means the standard readtable.
+      (assert (eq t (set-macro-character #\~ fun nil nil)))
+      (assert (eq fun (get-macro-character #\~ nil)))
+      (assert (eq t (set-dispatch-macro-character #\# #\~ fun nil)))
+      (assert (eq fun (get-dispatch-macro-character #\# #\~ nil))))))
+
+(defun cl-user::esoteric-load-thing ()
+  ;; This LOAD-AS-SOURCE will fail if PRINT1 reads as the keyword :PRIN1
+  (let ((s (make-string-input-stream
+            "(cl:in-package :cl-user) (prin1 'okey-dokey)")))
+    (let ((*package* *package*))
+      (sb-impl::load-as-source s :print t :verbose t))))
+
+(with-test (:name :reader-package-in-conditional)
+  ;; Sharp-plus binds *package* but not *reader-package* so that if,
+  ;; while reading the conditional expression itself, a read-time eval occurs
+  ;; expressly changing *package*, it should do whan you mean,
+  ;; though such usage is a little insane.
+  (let ((value
+         (read-from-string
+          "(#+#.(cl:progn (cl-user::esoteric-load-thing) 'sbcl) hiyya hoho)")))
+    (assert (equal value '(hiyya hoho)))))
+
 ;;; success
